// exported functions
var docusignOpenLoginModal;
var docusignOpenCreateModal;
var docusignOpenSelectAcctModal;
var docusignShowLoading;
var docusignHideLoading;


// below is only local to this file
(function($) {


var adjustModalForBwc = function(modalId) {
  var closeButton = $(modalId + ' a.close');
  closeButton.css('font-size', 'large');
  closeButton.css('position', 'absolute');
  closeButton.css('right', '20px');

  var title = $(modalId + ' h4');
  title.css('font-size', 'larger');

  var modalBody = $(modalId + ' .modal-body');
  modalBody.css('padding', '0');
  modalBody.css('padding-top', '15px');

  $(modalId + ' .modal-content').css('margin-left', '20px');
  $(modalId + ' .modal-footer').css('margin-top', '20px');
};


docusignOpenLoginModal = function(sugarUrl, dsEnv) {

  $.get('custom/docusign/html/login.html', function( html ) {

    addDSAccountDiv();
    $('#dsAccount').html( html );
    if (sugarUrl.isBwc) {
      adjustModalForBwc('#dsLoginModal');
    }
    $('#dsLoginModal').modal('show');

    $('#dsLoginOk').click(function() {

      docusignShowLoading();

      var dsEmail = $('#dsLoginEmail').val();
      var dsPassword = $('#dsLoginPassword').val();
      var errorMsg = '';
      if(dsEmail == '') errorMsg += 'Email';
      if(dsPassword == '') errorMsg += ', Password';
      if(errorMsg == '') {
        var emailReg = /^([\w-\.]+@([\w-]+\.)+[\w-]{2,4})?$/;
        if(!emailReg.test(dsEmail)) errorMsg += "Please enter a valid email: " + dsEmail;
      } else {
        errorMsg += ' field cannot be empty.';
        if(errorMsg.substring(0 ,2) == ', ')
           errorMsg = errorMsg.substring(2 ,errorMsg.length);
      }
      if(errorMsg != "") {
        docusignHideLoading();
        alert(errorMsg);
        return;
      }

      var data = {
        email: dsEmail,
        password: dsPassword,
        environment: dsEnv,
        dsAction: 'dsLogin',
      };

      $.post('index.php?entryPoint=docusignAccount', data, function(json) {

        docusignHideLoading();

        var response = JSON.parse(json);

        switch(response.status){

          case 'success':
            $('#dsLoginModal').modal('hide');
            saveDocusignConfigurationRecord(sugarUrl, dsEmail, response.dsOauthToken, dsEnv, response.accountId, response.baseUrl);
          break;

          case 'multiple_accounts':
            $('#dsLoginModal').modal('hide');
            docusignOpenSelectAcctModal(sugarUrl, dsEmail, response.dsOauthToken, dsEnv, response.accounts, response.baseUrl);
          break;

          case 'error':
            $('#dsLoginPassword').val('');
            if (/invalid_request: An OAuth2 error occurred./.test(response.errorMessage)) {
              // this means that user has reached oauth token limit
              alert('You have reached the maximum number of OAuth access tokens allowed for this account.\n\nPlease log into your account and revoke at least one token (found under Preferences -> Connected Apps).\n\nIf you need assistance, please contact DocuSign Support at <support@docusign.com>.');
            } else {
              alert(response.errorMessage);
            }
          break;

          default:
            alert(response.status);
          break;
        }
      });
    });

    $('#dsLoginCancel').click(function() {
      $('#dsLoginModal').modal('hide');
      window.location.reload(true);
    });

    $('#dsCreateLink').click(function() {
      $('#dsLoginModal').modal('hide');
      docusignOpenCreateModal(sugarUrl, dsEnv);
    });

  });

};


docusignOpenCreateModal = function(sugarUrl, dsEnv) {

  $.get('custom/docusign/html/create.html', function( html ) {

    addDSAccountDiv();
    $('#dsAccount').html( html );
    if (sugarUrl.isBwc) {
      adjustModalForBwc('#dsCreateModal');
    }
    $('#dsCreateModal').modal('show');

    // widen the dialog box
    $('#dsCreateModal').css({
      width: '600px',
      'margin-left': '-=100px',
    });

    $('#dsCreateOk').click(function() {
      docusignShowLoading();
      var company = $('#dsCreateCompanyName').val();
      var firstName = $('#dsCreateFirstName').val();
      var lastName = $('#dsCreateLastName').val();
      var email  = $('#dsCreateEmail').val();
      var password = $('#dsCreatePassword').val();
      var password2 = $('#dsCreatePassword2').val();
      var agreeTerm = $('#dsCreateAgreeTerm').is(':checked');
      var errorMsg = '';
      if(company == '') errorMsg += 'Company Name';
      if(firstName == '') errorMsg += ', First Name';
      if(lastName == '') errorMsg += ', Last Name';
      if(email == '') errorMsg += ', Email';
      if(password == '') errorMsg += ', Password';
      if(password2 == '') errorMsg += ', Repeat Password';
      if(errorMsg == '') {
        if( agreeTerm == false ){
          errorMsg += 'You must agree to the DocuSign Terms and Conditions to continue.';
        } else if (password != password2) {
          errorMsg += 'The passwords don\'t match.';
        } else {
          var emailReg = /^([\w-\.]+@([\w-]+\.)+[\w-]{2,4})?$/;
          if(!emailReg.test(email)) errorMsg += "Please enter a valid email: " + email;
        }
      } else {
        errorMsg += ' field cannot be empty.';
        if(errorMsg.substring(0 ,2) == ', ')
           errorMsg = errorMsg.substring(2 ,errorMsg.length);
      }
      if(errorMsg != "") {
        docusignHideLoading();
        alert(errorMsg);
        return;
      } 

      var data = {
        dsAction: 'dsCreate',
        company: company,
        firstName: firstName,
        lastName: lastName,
        email: email,
        password: password,
        environment: dsEnv,
        userSettings: [
          { name: 'allowSendOnBehalfOf', value: true },
          { name: 'apiAccountWideAccess', value: true },
          { name: 'canSendEnvelope', value: true },
        ],
      };

      $.post('index.php?entryPoint=docusignAccount', data, function(json) {
        var response = JSON.parse(json);
          docusignHideLoading();
          switch(response.status) {

            case 'success':
              $('#dsCreateModal').modal('hide');
              saveDocusignConfigurationRecord(sugarUrl, email, response.dsOauthToken, dsEnv, response.accountId, response.baseUrl);
              break;

            case 'error':
              $('#dsCreatePassword').val('');
              $('#dsCreatePassword2').val('');
              if (/^USER_ALREADY_EXISTS_IN_ACCOUNT/.test(response.errorMessage) ||
                  /^INVALID_PASSWORD/.test(response.errorMessage)) {
                alert('Username and email combination already exists for this account.');
              } else if (/invalid_request: An OAuth2 error occurred./.test(response.errorMessage)) {
                // this means that user has reached oauth token limit
                alert('You have reached the maximum number of OAuth access tokens allowed for this account.\n\nPlease log into your account and revoke at least one token (found under Preferences -> Connected Apps).\n\nIf you need assistance, please contact DocuSign Support at <support@docusign.com>.');
              } else {
                alert(response.errorMessage);
              }
              break;

            default:
              alert(response.status);
              break;
          }
      });
    });

    $('#dsCreateCancel').click(function() {
      $('#dsCreateModal').modal('hide');
      window.location.reload(true);
    });

    $('#dsLoginLink').click(function() {
      $('#dsCreateModal').modal('hide');
      docusignOpenLoginModal(sugarUrl, dsEnv);
    });

  });

};


docusignOpenSelectAcctModal = function(sugarUrl, dsEmail, dsPassword, dsEnv, accounts, baseUrl) {

  $.get('custom/docusign/html/select_account.html', function( html ) {
    addDSAccountDiv();
    $('#dsAccount').html( html );

    for (var i in accounts) {
      var account = accounts[i];
      var dropdown = $('<option>', { value: account.id }).appendTo('#dsAccountDropdown');
      dropdown.text(account.name + ' - ' + account.id + (account.isDefault ? ' (Default)' : ''));
    }

    if (sugarUrl.isBwc) {
      adjustModalForBwc('#dsSelectAcctModal');
    }
    $('#dsSelectAcctModal').modal('show');
    $('#dsSelectAcctOk').click(function() {
      saveDocusignConfigurationRecord(sugarUrl, dsEmail, dsPassword, dsEnv, $('#dsAccountDropdown').val(), baseUrl);
      $('#dsSelectAcctModal').modal('hide');
    });
    
  });

};


docusignShowLoading = function() {
  if( $('#dsLoading').length == 0 ){
    $('.alert-top').append('<div id="dsLoading" class="alert alert-process"><strong>Please wait</strong><div class="loading"><span class="l1"></span><span class="l2"></span><span class="l3"></span></div></div>');
  }
}


docusignHideLoading = function() {
  if( $('#dsLoading').length > 0 ){
    $('#dsLoading').remove();
  }
}


function addDSAccountDiv() {
  if ($('#dsAccount').length === 0) {
    $('<div id="dsAccount" style="vertical-align:middle;"></div>').appendTo('body');
  }
}


function saveDocusignConfigurationRecord(sugarUrl, email, password, environment, accountId, baseUrl) {
  $.ajax({
    type: 'GET',
    url: sugarUrl.apiRoot + '/docus_DocuSign_Configuration',
    headers: {
      'Content-Type': 'application/json',
      'OAuth-Token': SUGAR.App.api.getOAuthToken(),
    },
    success: function(response){
      var recordId = '';
      var requestType = 'POST';

      if(response.records.length > 0){ 
        requestType = 'PUT';
        recordId = response.records[0].id;
      } 

      $.ajax({
        type: requestType,
        url: sugarUrl.apiRoot + '/docus_DocuSign_Configuration/' + recordId,
        headers: {
          'Content-Type': 'application/json',
          'OAuth-Token': SUGAR.App.api.getOAuthToken(),
        },
        data: JSON.stringify({
          name: 'DocuSignConfig',
          docusign_email: email,
          docusign_password: password,
          docusign_environment: environment,
          docusign_account_id: accountId,
          docusign_base_url: baseUrl,
        }),
        success: function() {
          window.top.location.reload(true);
        },
        error: function() {
          alert('Failed to configure DocuSign account...');
        },
      });
    }
  });

}


})(jQuery);
