<?php

if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once 'custom/docusign/util.php';
require_once 'custom/docusign/dsapi/library/src/DocuSign_Client.php';
require_once 'custom/docusign/dsapi/library/src/service/DocuSign_AccountService.php';


$dsAction = $_REQUEST['dsAction'];

if( empty($dsAction) ){
  echo json_encode(array(
   'status' => 'bad_request',
  ));
  return;
}

if( $dsAction == 'dsLogin' ) {

  if( empty($_REQUEST['email']) ||
      empty($_REQUEST['password']) ||
      empty($_REQUEST['environment']) ){
      echo json_encode(array(
          'status' => 'bad_request',
      ));
      return;
  }

  $creds = array(
      'integrator_key' => DocuSignUtil::access(),
      'email' => $_REQUEST['email'],
      'password' => $_REQUEST['password'],
      'version' => 'v2',
      'environment' => $_REQUEST['environment'],
  );

  $client = new DocuSign_Client($creds);

  if( $client->hasError() ){
   
    // Authentication failed
    echo json_encode(array(
        'status' => 'error',
        'errorMessage' => $client->getErrorMessage()
    ));

  } else if ( $client->hasMultipleAccounts() ) {

    // Multiple accounts found
    $response = $client->authenticate();
    $accounts = array();
    foreach ($response->loginAccounts as $loginAccount) {
      $account = array();
      $account['id'] = $loginAccount->accountId;
      $account['name'] = $loginAccount->name;
      $account['isDefault'] = $loginAccount->isDefault === 'true';
      $accounts[] = $account;
    }

    try {
        $token = $client->createOauthToken();
    } catch (DocuSign_IOException $e) {
        echo json_encode(array(
            'status' => 'error',
            'errorMessage' => $e->getMessage(),
        ));
        return;
    }

    echo json_encode(array(
        'status' => 'multiple_accounts',
        'accounts' => $accounts,
        'dsOauthToken' => $token,
        'baseUrl' => $client->getBaseURL(),
    ));

  } else {

    try {
        $token = $client->createOauthToken();
    } catch (DocuSign_IOException $e) {
        echo json_encode(array(
            'status' => 'error',
            'errorMessage' => $e->getMessage(),
        ));
        return;
    }

    // Single account found
    echo json_encode(array(
        'status' => 'success',
        'accountId' => $client->getAccountID(),
        'dsOauthToken' => $token,
        'baseUrl' => $client->getBaseURL(),
    ));

  }


} else if( $dsAction == 'dsCreate' ) {

  if( empty($_REQUEST['company']) ||
      empty($_REQUEST['firstName']) ||
      empty($_REQUEST['lastName']) ||
      empty($_REQUEST['email']) ||
      empty($_REQUEST['password']) ||
      empty($_REQUEST['environment']) ){ 
     echo json_encode(array(
          'status' => 'bad_request',
      ));
      return;
  }

  $creds = array(
      'integrator_key' => DocuSignUtil::access(),
      'email' => '',
      'password' => '',
      'version' => 'v2',
      'environment' => $_REQUEST['environment'],
  );

  $client = new DocuSign_Client($creds);
  $dsService = new DocuSign_AccountService($client);

  try {
    $response = $dsService->account->getAccountProvisioning('CRM_SOBO');
  } catch(DocuSign_IOException $e) {
    echo json_encode(array(
       'status' => 'error',
       'errorMessage' => $e->getMessage(),
    ));
    return;
  }

  $distributorCode = $response->distributorCode;
  $distributorPassword = $response->distributorPassword;
  $planId = $response->defaultPlanId;
  $initialUser = new DocuSign_InitialUser($_REQUEST['email'], 
                                          $_REQUEST['firstName'], 
                                          $_REQUEST['lastName'], 
                                          $_REQUEST['firstName'] . ' ' . $_REQUEST['lastName'], 
                                          $_REQUEST['password']);
  $referralInformation = new DocuSign_ReferralInformation('Sugar7', 'Sugar7');
  
  try {
    $response = $dsService->account->createAccount( $_REQUEST['company'],
                                                    $distributorCode,
                                                    $distributorPassword,
                                                    $planId,
                                                    $initialUser,
                                                    $referralInformation);
  } catch(DocuSign_IOException $e) {
    echo json_encode(array(
       'status' => 'error',
       'errorMessage' => $e->getMessage(),
    ));
    return;
  }

  // we have to reinitiate the client in order to generate oauth token
  $creds = array(
      'integrator_key' => DocuSignUtil::access(),
      'email' => $_REQUEST['email'],
      'password' => $_REQUEST['password'],
      'version' => 'v2',
      'environment' => $_REQUEST['environment'],
  );
  $client = new DocuSign_Client($creds);

  try {
      $token = $client->createOauthToken();
  } catch (DocuSign_IOException $e) {
      echo json_encode(array(
          'status' => 'error',
          'errorMessage' => $e->getMessage(),
      ));
      return;
  }

  echo json_encode(array(
        'status' => 'success',
        'accountId' => $response->accountId,
        'dsOauthToken' => $token,
        'baseUrl' => $client->getBaseURL(),
  ));


} else if( $dsAction == 'dsCreateUser' ) {

  $creds = DocuSignUtil::getDocuSignCreds();
  $client = new DocuSign_Client($creds);
  $dsService = new DocuSign_AccountService($client);

  $user = new DocuSign_InitialUser(
    $_REQUEST['email'],
    $_REQUEST['firstName'],
    $_REQUEST['lastName'],
    $_REQUEST['firstName'] . ' ' . $_REQUEST['lastName'],
    $_REQUEST['password'],
    $_REQUEST['userSettings']
  );
 
  try {
    $response = $dsService->account->createUserAccount($user);
  } catch(DocuSign_IOException $e) {
    echo json_encode(array(
       'status' => 'error',
       'errorMessage' => $e->getMessage(),
    ));
    return;
  }

  echo json_encode(array(
        'status' => 'success',
        'userId' => $response->newUsers[0]->userId,
  ));
}


