<?php

class SendWithDocusignHook
{
    public function renderEntry($event, $arguments)
    {
        if ($_REQUEST['action'] !== 'DetailView') {
            return; // don't execute this logic hook if we don't have to
        }

        $code = <<<'EOC'
<script>
jQuery(document).ready(function($) {
  function docusignGetFile(url, callback) {
    var req = new XMLHttpRequest();
    req.open('GET', url, true);
    req.responseType = 'blob';

    var _arrayBufferToBase64 = function(buffer) {
      var binary = '';
      var octets = new Uint8Array(buffer);

      $.each(octets, function(index, octet) {
        binary += String.fromCharCode(octet);
      });

      return window.btoa(binary);
    }

    var _blobToBase64 = function(blob, callback) {
      var reader = new FileReader();

      reader.addEventListener('loadend', function() {
        callback(_arrayBufferToBase64(reader.result));
      });

      reader.readAsArrayBuffer(blob);
    }

    req.onload = function() {
      _blobToBase64(req.response, function(base64) {
        callback(base64);
      });
    };

    req.send();
  }


  // Make sure we get Sugar7 functionality, or else DS iframe will not work.
  SUGAR.App = window.top.SUGAR.App;


  // load utility scripts and proceed
  $.getScript('custom/docusign/sugar.js', function() {
    $.getScript('https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.1.0/moment.min.js', function() {
      $.getScript('https://cdnjs.cloudflare.com/ajax/libs/jstimezonedetect/1.0.4/jstz.min.js', function() {
        $.getScript('custom/docusign/load.js', function() {
          docusignBwcMain(new DocusignSugarUrl(window.top.location.href));
        });
      });
    });
  });


  function docusignBwcMain(sugarUrl) {
    var dsEntry = '<li><a id="docusignButton">Send with DocuSign</a></li>';
    $('.actionsContainer ul.subnav').append(dsEntry);

    docusignCreateStatusTable(sugarUrl);


    // Is the user an Administrator?
    // This concerns account creation and SOBO usage.
    var isAdmin;
    $.ajax({
      type: 'GET',
      url: sugarUrl.apiRoot + '/me',
      headers: {
        'Content-Type': 'application/json',
        'OAuth-Token': SUGAR.App.api.getOAuthToken(),
      },
      async: false,
      success: function(data) {
        isAdmin = data.current_user.type === 'admin';
      }
    });


    $.getScript('custom/docusign/topshelf.js', function() {
      docusignCreateFrame(true);

      $('#docusignButton').click(function(e) {
        e.preventDefault();


        var docusignConfig;
        var docusignEnvironment;
        $.ajax({
          type: 'GET',
          url: sugarUrl.apiRoot + '/docus_DocuSign_Configuration',
          headers: {
            'Content-Type': 'application/json',
            'OAuth-Token': SUGAR.App.api.getOAuthToken(),
          },
          async: false,
          success: function(data) {
            docusignConfig = data.records[0];
            docusignEnvironment = (typeof docusignConfig === 'undefined') ? 'www' : docusignConfig.docusign_environment;
          }
        });

        // if DS master account has not been set up yet
        if (typeof docusignConfig === 'undefined' || docusignConfig.docusign_email === '') {
          if (isAdmin) {
            $.getScript('custom/docusign/account.js', function() {
              docusignOpenLoginModal(sugarUrl, docusignEnvironment);
            });
          } else {
            alert('SugarCRM is not configured with DocuSign. Please contact your administrator.');
          }
          return;
        }


        var quotePdfUrl = $('#Invoice_pdfview').attr('href').replace(/.*(\/index.php\?.*)/, sugarUrl.root + '$1');
        docusignGetFile(quotePdfUrl, function(pdf) {
          var quoteDoc = {
            name: 'Quote',
            content: pdf,
          };

          docusignOpenFrame(true);
          setTimeout(function() {
            docusignLoadView(sugarUrl, [quoteDoc], [], isAdmin);
          }, 1000);
        });
      });
    });
  }


  function docusignCreateStatusTable(sugarUrl) {
    var table = $('<li id="whole_subpanel_dsStatuses" class="noBullet"></li>').appendTo('#subpanel_list');
    $.ajax({
      type: 'GET',
      url: 'custom/docusign/html/subpanel.html',
      async: false,
      success: function(data) {
        table.html(data);
      },
    });
    table.find('*').removeAttr('id'); // clear all copied IDs

    var title = table.find('h3 span');
    title.find('.utilsLink').removeAttr('onclick');
    title.find('.utilsLink').removeAttr('href');
    title.html(title.html().replace(/(.*<\/span>).*/, '$1 DocuSign Statuses'));

    table.find('.SugarActionMenu').remove();

    var header = table.find('tr[height=20]:first');
    header.html(''); // clear previous data
    header.append('<th width="2%" scope="col"></th>');
    header.append('<th width="30%" scope="col">Envelope ID</th>');
    header.append('<th width="30%" scope="col">Envelope Status</th>');
    header.append('<th width="30%" scope="col">Date Modified</th>');
    header.append('<th width="8%" scope="col"></th>');

    var body = table.find('table.view');
    body.find('tr:gt(2)').remove(); // clear previous data

    var sugarUserId = docusignLoadSugarUserId(sugarUrl);

    var statusRecords;
    $.ajax({
      type: 'GET',
      url: sugarUrl.apiRoot + '/docus_DocuSign_Status',
      headers: {
        'Content-Type': 'application/json',
        'OAuth-Token': SUGAR.App.api.getOAuthToken(),
      },
      data: { max_num: 9999 },
      async: false,
      success: function(data) {
        statusRecords = data.records.filter(function(dsStatus) {
          return sugarUrl.isStatusOurs(sugarUserId, dsStatus);
        });
      },
    });

    $.each(statusRecords, function(index, statusRecord) {
      var stat = statusRecord.docusign_envelopeStatus;
      var envelopeStatus = (stat === 'completed') ? '<a href="' + sugarUrl.root + '#bwc/index.php?module=Documents&action=DetailView&record=' + statusRecord.description + '" target="_blank">completed</a>' : stat;

      var row = $('<tr class="dsStatus" height="30"></tr>');
      row.append('<td scope="row"><span></span></td>');
      row.append('<td scope="row"><span>' + statusRecord.name + '</span></td>');
      row.append('<td scope="row"><span>' + envelopeStatus + '</span></td>');
      row.append('<td scope="row"><span>' + docusignLoadDate(statusRecord.date_modified) + '</span></td>');
      row.append('<td scope="row"><span></span></td>');
      body.append(row);
    });

    var statLength = statusRecords.length;
    if (statLength > 0) {
      table.find('.pageNumbers').text('(1 - ' + statLength + ' of ' + statLength + ')');
    }

    if (statusRecords.length === 0) {
      var row = $('<tr class="oddListRowS1" height="20"></tr>');
      row.append('<td colspan="5" style="padding-left: 10px;"><em>No data</em></td>');
      body.append(row);
    }

    // Yes, these are reversed.
    body.find('.dsStatus:odd').addClass('evenListRowS1');
    body.find('.dsStatus:even').addClass('oddListRowS1');
  }
});
</script>
EOC;


        // output the code to Sugar
        echo $code;
    }
}

