/*
 * You will need these statements prior to importing this file:
 *
   $.getScript('https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.1.0/moment.min.js', function() {
     $.getScript('https://cdnjs.cloudflare.com/ajax/libs/jstimezonedetect/1.0.4/jstz.min.js', function() {
       ...
     });
   });
 *
 */

// exported functions
var docusignLoadView;
var docusignLoadStatuses;
var docusignLoadDate;
var docusignLoadSugarUserId;


// below is only local to this file
(function($) {


function getIframeUrl(sugarUrl, isAdmin, payload) {
  if (isAdmin)
    return getSenderView(payload);


  var isInternetExplorer = /*@cc_on!@*/false;

  var info = getSavedInfo(sugarUrl);
  var userEmail = info.email;
  var origFullName = info.fullName;

  var savedFullName = getSavedFullName(sugarUrl);
  var fullName = (savedFullName !== '') ? savedFullName : origFullName;
  fullName = checkForSpaceInName(sugarUrl, fullName);

  // if user is not an admin, do the SOBO user process
  while (true) {
    try {
      createUserAccount(userEmail, fullName, payload.email, payload.password, payload.environment);
      break; // if no exception thrown, exit from loop
    } catch (e) {
      if (/^USER_ALREADY_EXISTS_IN_ACCOUNT/.test(e.message)) {
        // this is fine; exit from loop
        break;
      } else if (/^USER_AUTHENTICATION_FAILED/.test(e.message)) {
        if (isInternetExplorer) {
          // need to shorten message due to IE's fixed prompt size
          fullName = prompt('DS account already exists with this name: ' + fullName + '\nEnter a different name to proceed:');
        } else {
          fullName = prompt('You have an existing DocuSign account with the following information:\nEmail: ' + userEmail + '\nName: ' + fullName + '\n\nPlease enter a different name:');
        }
        if (fullName === null || fullName === '') {
          throw new Error('CANCEL');
        }

        fullName = checkForSpaceInName(sugarUrl, fullName);
      } else {
        throw e; // propogate other errors upward
      }
    }
  }

  return getSenderView(payload, userEmail);
}


// START helper functions for getIframeUrl()

// retrieve user info from Sugar
function getSavedInfo(sugarUrl) {
  var info = {};

  $.ajax({
    type: 'GET',
    url: sugarUrl.apiRoot + '/Users',
    headers: {
      'Content-Type': 'application/json',
      'OAuth-Token': SUGAR.App.api.getOAuthToken(),
    },
    data: { filter: [{ id: docusignLoadSugarUserId(sugarUrl) }] },
    async: false,
    success: function(data) {
      info.email = data.records[0].email1;
      info.fullName = data.records[0].full_name;
    },
  });

  return info;
}

// returns empty string if this field has not been set yet
function getSavedFullName(sugarUrl) {
  var fullName;

  $.ajax({
    type: 'GET',
    url: sugarUrl.apiRoot + '/me/preference/docusign_full_name',
    headers: {
      'Content-Type': 'application/json',
      'OAuth-Token': SUGAR.App.api.getOAuthToken(),
    },
    async: false,
    success: function(data) {
      fullName = data;
    },
  });

  return fullName;
}

function setSavedFullName(sugarUrl, fullName) {
  $.ajax({
    type: 'PUT',
    url: sugarUrl.apiRoot + '/me/preferences',
    headers: {
      'Content-Type': 'application/json',
      'OAuth-Token': SUGAR.App.api.getOAuthToken(),
    },
    data: JSON.stringify({ docusign_full_name: fullName }),
    async: false,
  });
}

function checkForSpaceInName(sugarUrl, name) {
  if (name === null)
    throw new Error('CANCEL');

  // DS API rejects single names, so we have to ask the user again
  while (!(/\s+/.test(name))) {
    name = prompt('Your name only consists of one name.\n\nPlease enter a full name (e.g. Jane Doe):');

    if (name === null) // user clicks on Cancel
      throw new Error('CANCEL');
  }

  setSavedFullName(sugarUrl, name);

  // looks good; return the name
  return name;
}

function createUserAccount(userEmail, fullName, masterEmail, masterPassword, environment) {
  var m = fullName.match(/(.*)\s+(.*)/);
  var firstName = m[1];
  var lastName = m[2];

  var data;
  $.ajax({
    type: 'POST',
    url: 'index.php?entryPoint=docusignAccount',
    data: {
      dsAction: 'dsCreateUser',
      masterEmail: masterEmail,
      masterPassword: masterPassword,
      firstName: firstName,
      lastName: lastName,
      email: userEmail,
      password: Math.random().toString(36).slice(2), // randomized string
      environment: environment,
      userSettings: [
        { name: 'canSendEnvelope', value: true },
      ],
    },
    async: false,
    success: function(json) {
      data = JSON.parse(json);
    },
  });

  if (data.status === 'error')
    throw new Error(data.errorMessage);
}

function getSenderView(payload, soboUser) {
  if (typeof soboUser === 'undefined')
    payload.soboUser = null;
  else
    payload.soboUser = soboUser;

  var data;
  return $.ajax({
    type: 'POST',
    url: 'index.php?entryPoint=docusignView',
    data: payload,
    success: function(json) {
      data = JSON.parse(json);
    },
  });
}

// END helper functions for getIframeUrl()


docusignLoadView = function(sugarUrl, prependedDocs, appendedDocs, isAdmin) {
  // set default values
  if (typeof prependedDocs === 'undefined' || prependedDocs === null)
    prependedDocs = [];
  if (typeof appendedDocs === 'undefined' || appendedDocs === null)
    appendedDocs = [];


  var contactsUrl;
  if (sugarUrl.recordType === 'Contacts' || sugarUrl.recordType === 'Leads') { // has contact record embedded
    contactsUrl = sugarUrl.apiParentRecord;
  } else if (sugarUrl.recordType === 'Quotes') {
    var accountId;
    $.ajax({
      type: 'GET',
      url: sugarUrl.apiParentRecord,
      headers: {
        'Content-Type': 'application/json',
        'OAuth-Token': SUGAR.App.api.getOAuthToken(),
      },
      async: false,
      success: function(data) {
        accountId = data.account_id;
      },
    });

    contactsUrl = sugarUrl.apiRoot + '/Accounts/' + accountId + '/link/contacts';
  } else { // has related contact records
    contactsUrl = sugarUrl.apiParentRecord + '/link/contacts';
  }

  var contactRecords = [];
  $.ajax({
    url: contactsUrl,
    headers: {
      'Content-Type': 'application/json',
      'OAuth-Token': SUGAR.App.api.getOAuthToken(),
    },
    async: false,
    success: function(data) {
      if ('email1' in data) {
        contactRecords = [{
          name: data.name,
          email1: data.email1,
        }];
      } else {
        contactRecords = data.records;
      }
    },
  });

  var documentsUrl = sugarUrl.apiParentRecord + '/link/documents';
  var documentRecords = [];
  $.ajax({
    url: documentsUrl,
    headers: {
      'Content-Type': 'application/json',
      'OAuth-Token': SUGAR.App.api.getOAuthToken(),
    },
    async: false,
    success: function(data) {
      documentRecords = data.records;
    },
  });

  var contacts = [];
  $.each(contactRecords, function(index, contactRecord) {
    contacts.push({
      name: contactRecord.name,
      email: contactRecord.email1,
    });
  });

  var documents = [];
  $.each(documentRecords, function(index, documentRecord) {
    $.ajax({
      type: 'GET',
      url: sugarUrl.apiRoot + '/Documents/' + documentRecord.id,
      async: false,
      headers: {
        'Content-Type': 'application/json',
        'OAuth-Token': SUGAR.App.api.getOAuthToken(),
      },
      success: function(data) {
        var docName = (data.name === '') ? data.filename : data.name;

        documents.push({
          name: docName.replace("'", '_'),
          revisionId: data.document_revision_id,
        });
      },
    });
  });

  var returnUrlParams = {
    urlRoot: sugarUrl.root,
    apiRoot: sugarUrl.apiRoot,
    parentRecord: sugarUrl.recordType,
    parentId: sugarUrl.recordId,
    token: SUGAR.App.api.getOAuthToken(),
    sugarUserId: docusignLoadSugarUserId(sugarUrl),
    bwc: sugarUrl.isBwc,
  };
  var payload = {
    sugarUrlRoot: sugarUrl.root,
    returnUrl: sugarUrl.root + 'custom/docusign/dsapi/return.php?' + $.param(returnUrlParams),
    contacts: contacts,
    documents: prependedDocs.concat(documents),
  };

  $.when(getIframeUrl(sugarUrl, isAdmin, payload)).done(function(json) {
    var data = JSON.parse(json);

    if ('error' in data) {
      if (data.error === 'CANCEL') {
        // cancel loading DS iframe and return to Sugar
      } else if (/^USER_DOES_NOT_EXIST_IN_SYSTEM/.test(data.error) ||
                 /^INVALID_REQUEST_PARAMETER/.test(data.error)) {
        alert('The DocuSign account connected to your Sugar 7 environment has been closed. Please contact your system administrator to reset your DocuSign account credentials.');
      } else {
        alert('DocuSign error: ' + data.error);
      }

      docusignCloseFrame();
      return;
    }

    var context = sugarUrl.isBwc ? window.top.document : document;
    $('#docusignFrame', context).attr('src', data.url);
  });
};


docusignLoadStatuses = function(statusData, sugarUrl, sugarUserId) {
  var statuses = {
    created: 0,
    sent: 0,
    delivered: 0,
    completed: 0,
    declined: 0,
    voided: 0,
  };
  var colors = {
    created: '#F8F340',
    sent: '#6EE5FD',
    delivered: '#735E8C',
    completed: '#64D56C',
    declined: '#FF7400',
    voided: '#EC6B51',
  };
  var labels = {
    created: 'Created',
    sent: 'Sent',
    delivered: 'Delivered',
    completed: 'Completed',
    declined: 'Declined',
    voided: 'Voided',
  };

  var total = 0;
  statusData.records.filter(function(dsStatus) {
    if (sugarUrl.isStatusOurs(sugarUserId, dsStatus)) {
      statuses[dsStatus.docusign_envelopeStatus]++;
      total++;
    }
  });

  if (total === 0) {
    $('#docusignChart').text('No envelope statuses found.');
  } else {
    $.plot($('#docusignChart'), [
      { label: labels.voided, color: colors.voided, data: statuses.voided },
      { label: labels.declined, color: colors.declined, data: statuses.declined },
      { label: labels.completed, color: colors.completed, data: statuses.completed },
      { label: labels.delivered, color: colors.delivered, data: statuses.delivered },
      { label: labels.sent, color: colors.sent, data: statuses.sent },
      { label: labels.created, color: colors.created, data: statuses.created },
    ], {
      series: {
        pie: {
          show: true,
          radius: 0.65,
          innerRadius: 0.25,
          label: {
            show: true,
            formatter: function(label, series) {
              return $('<div />').css('text-align', 'center').text(label).html();
            },
          },
          stroke: {
            width: 0,
          },
        },
      },
      legend: {
        show: false,
      },
    });
  }

  $('#docusignInfo').empty(); // make sure this is cleared first
  $.each([
    'created',
    'sent',
    'delivered',
    'completed',
    'declined',
    'voided',
  ], function(index, element) {
    var span = $('<span />');
    span.attr('id', element);
    span.css('background', colors[element]);
    span.text(statuses[element]);

    var cell = $('<td />');
    span.appendTo(cell);
    $('<br />').appendTo(cell);
    cell.append(labels[element]);
    cell.appendTo('#docusignInfo');
  });
  $('#docusignInfo td').css('width', '17%');
  $('#docusignInfo span').css({
    margin: 'auto',
    'margin-bottom': '5px',
    cursor: 'default',
  });
  $('#docusignInfo span').addClass('btn');
  $('#docusignInfo #delivered').css('color', 'white');
  $('#docusignInfo #declined').css('color', 'white');
  $('#docusignInfo #voided').css('color', 'white');
};


// this function converts a raw timestamp from Sugar into HTML format
docusignLoadDate = function(timestamp) {
  var time = moment(timestamp);
  var simple = time.format('MMM D YYYY, h:mm A');
  var full = time.format('dddd, MMMM D, YYYY, h:mm A') + ' (' + jstz.determine().name() + ')';
  return '<div class="docusignEllipsis" rel="tooltip" data-original-title="' + full + '" style="white-space: nowrap; overflow: hidden; text-overflow: ellipsis;">' + simple + '</div>';
};


docusignLoadSugarUserId = function(sugarUrl) {
  var sugarUserId;

  $.ajax({
    type: 'GET',
    url: sugarUrl.apiRoot + '/me',
    headers: {
      'Content-Type': 'application/json',
      'OAuth-Token': SUGAR.App.api.getOAuthToken(),
    },
    async: false,
    success: function(data) {
      sugarUserId = data.current_user.id;
    },
  });

  return sugarUserId;
};


})(jQuery);


/*// MOCHA
// export public functions to the Mocha unit test framework
exports.loadView = docusignLoadView;
exports.loadStatuses = docusignLoadStatuses;
exports.loadDate = docusignLoadDate;
exports.loadSugarUserId = docusignLoadSugarUserId;
*/// MOCHA


