(function($) {


// DS dashlet globals
var dsStatusData = null;


// start loading the dashlet
$.getScript('custom/docusign/sugar.js', function() {
  prepareDashlet(new DocusignSugarUrl(window.location.href));
});


// try to get DS data first to see if DS connector is disabled
function prepareDashlet(sugarUrl) {
  $.ajax({
    type: 'GET',
    url: sugarUrl.apiRoot + '/docus_DocuSign_Status',
    headers: {
      'Content-Type': 'application/json',
      'OAuth-Token': SUGAR.App.api.getOAuthToken(),
    },
    data: { max_num: 9999 },
    success: function(data) {
      dsStatusData = data;
    },
    error: function() {
      // getting a 404 error means that the connector is disabled
      dsStatusData = 'DISABLED';
    },
  });

  SUGAR.App.utils.doWhen(function() {
    return dsStatusData !== null;
  }, function() { // then do this
    if (dsStatusData === 'DISABLED') {
      $('#docusignHeader').hide();
    } else {
      loadScripts(sugarUrl);
    }
  });
}


// load JS libraries and go into the main code
function loadScripts(sugarUrl) {
  $.getScript('https://cdnjs.cloudflare.com/ajax/libs/flot/0.8.1/jquery.flot.min.js', function() {
    $.getScript('custom/docusign/jquery.flot.pie.js', function() {
      $.getScript('https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.1.0/moment.min.js', function() {
        $.getScript('https://cdnjs.cloudflare.com/ajax/libs/jstimezonedetect/1.0.4/jstz.min.js', function() {
          $.getScript('custom/docusign/load.js', function() {
            main(sugarUrl);
          });
        });
      });
    });
  });
}


function main(sugarUrl) {

  // Is the user an Administrator?
  // This concerns account creation and SOBO usage.
  var isAdmin;
  $.ajax({
    type: 'GET',
    url: sugarUrl.apiRoot + '/me',
    headers: {
      'Content-Type': 'application/json',
      'OAuth-Token': SUGAR.App.api.getOAuthToken(),
    },
    async: false,
    success: function(data) {
      isAdmin = data.current_user.type === 'admin';
    },
    error: function() {
      // session has been active for too long, so just refresh page
      window.top.location.reload(true);
    },
  });

  var docusignConfig;
  var docusignEnvironment;
  $.ajax({
    type: 'GET',
    url: sugarUrl.apiRoot + '/docus_DocuSign_Configuration',
    headers: {
      'Content-Type': 'application/json',
      'OAuth-Token': SUGAR.App.api.getOAuthToken(),
    },
    async: false,
    success: function(data) {
      docusignConfig = data.records[0];
      docusignEnvironment = (typeof docusignConfig === 'undefined') ? 'www' : docusignConfig.docusign_environment;
    }
  });


  // if DS master account has not been set up yet
  if (typeof docusignConfig === 'undefined' || docusignConfig.docusign_email === '') {
    var notSetUp = isAdmin ? '<a style="font-weight: bold">Configure DocuSign</a>' : '<span>SugarCRM is not configured with DocuSign. Please contact your administrator.</span>';
    var configLink = $(notSetUp).appendTo('#docusignHeader');
    if (isAdmin) {
      configLink.parent().css('text-align', 'center');

      $.getScript('custom/docusign/account.js', function() {
        configLink.click(function() {
          docusignOpenLoginModal(sugarUrl, docusignEnvironment);
        });
      });
    }

    return;
  }


  // set up the DS dashlet header
  $('#docusignHeader').css('height', '40px');

  // get the current record's title name
  var recordName;
  $.ajax({
    type: 'GET',
    url: sugarUrl.apiParentRecord,
    headers: {
      'Content-Type': 'application/json',
      'OAuth-Token': SUGAR.App.api.getOAuthToken(),
    },
    async: false,
    success: function(data) {
      recordName = data.name;
    },
  });

  var headerText;
  if (sugarUrl.isAtHome)
    headerText = 'Summary of Your DocuSign Envelope Statuses for All Objects';
  else if (sugarUrl.isListing)
    headerText = 'Your DocuSign Envelope Statuses for All ' + sugarUrl.recordType;
  else
    headerText = 'All DocuSign Envelope Statuses for ' + recordName;

  var docusignHeaderText = $('<div id="docusignHeaderText"></div>').appendTo('#docusignHeader');
  docusignHeaderText.css({
    display: 'inline-block',
    position: 'absolute',
    'font-weight': 'bold',
  });
  docusignHeaderText.text(headerText);


  // set up the DS button
  if (!sugarUrl.isListing && !sugarUrl.isAtHome) {
    $('<button id="docusignButton" class="btn btn-primary"></button>').appendTo('#docusignHeader');
    $('#docusignButton').css('float', 'right');
    $('#docusignButton').text('Send with DocuSign');

    docusignHeaderText.css('width', '60%'); // make room for button

    $.getScript('custom/docusign/topshelf.js', function() {
      docusignCreateFrame();

      $('#docusignButton').click(function() {
        docusignOpenFrame();

        setTimeout(function() {
          docusignLoadView(sugarUrl, [], [], isAdmin);
        }, 1000);
      });
    });
  }


  $('#docusignChart').click(function() {
    $('#docusignChart').css('display', 'none');
    $('#docusignTable').css('display', 'block'); // show
  });

  $('#docusignBack').click(function(e) {
    e.preventDefault();

    $('#docusignChart').css('display', 'block'); // show
    $('#docusignTable').css('display', 'none');
  });


  var sugarUserId = docusignLoadSugarUserId(sugarUrl);
  docusignLoadStatuses(dsStatusData, sugarUrl, sugarUserId);


  // load data into table view
  $('#docusignChart').click(function() {
    $.ajax({
      type: 'GET',
      url: sugarUrl.apiRoot + '/docus_DocuSign_Status',
      headers: {
        'Content-Type': 'application/json',
        'OAuth-Token': SUGAR.App.api.getOAuthToken(),
      },
      data: { max_num: 9999 },
      success: function(data) {

        var database = data.records.filter(function(dsStatus) {
          return sugarUrl.isStatusOurs(sugarUserId, dsStatus);
        });

        var records = [];
        $.each(database, function(index, value) {
          records.push([
            value.name, // i.e. DS envelope ID
            value.docusign_envelopeStatus,
            value.date_modified,
            value.description, // i.e. Sugar document ID
          ]);
        });

        var oTable = $('#docusignTable table').dataTable({
          aaData: records,
          aaSorting: [[2, 'desc']],
          aoColumnDefs: [{
            mDataProp: function(data, type) {
              var envId = data[0];
              if (type !== 'display')
                return envId;

              return '<div class="docusignEllipsis" rel="tooltip" data-original-title="' + envId + '" style="white-space: nowrap; overflow: hidden; text-overflow: ellipsis;">' + envId + '</div>';
            },
            aTargets: [0],
          }, {
            mDataProp: function(data, type) {
              var envStatus = data[1];
              if (type !== 'display' || envStatus !== 'completed')
                return envStatus;

              var documentId = data[3];
              return '<a href="' + sugarUrl.root + '#bwc/index.php?module=Documents&action=DetailView&record=' + documentId + '" target="_blank">' + envStatus + '</a>';
            },
            aTargets: [1],
          }, {
            mDataProp: function(data, type) {
              var timestamp = data[2];
              if (type !== 'display')
                return timestamp;

              return docusignLoadDate(timestamp);
            },
            aTargets: [2],
          }],
          bDestroy: true,
          bFilter: false,
          bLengthChange: false,
          bInfo: false,
          iDisplayLength: 4,
          fnDrawCallback: function() {
            function applyButtonStyle(element, isNext, isDisabled) {
              element.css({
                position: 'absolute',
                bottom: 0,
                'text-decoration': 'underline',
              });

              if (isNext)
                element.css('right', '25px');
              else
                element.css('right', '75px');

              if (isDisabled)
                element.css('color', 'gray');
              else
                element.css('color', $('#docusignBack').css('color'));
            }

            applyButtonStyle($('#docusignTable .paginate_disabled_previous'), false, true);
            applyButtonStyle($('#docusignTable .paginate_disabled_next'), true, true);
            applyButtonStyle($('#docusignTable .paginate_enabled_previous'), false, false);
            applyButtonStyle($('#docusignTable .paginate_enabled_next'), true, false);

            $('#docusignTable table').css('width', '100%');

            var cellWidth = $('#docusignTable td').width() * 0.80;
            $('.docusignEllipsis').css('width', cellWidth);
          },
          oLanguage: { sEmptyTable: 'No envelope statuses found.' },
        });
        oTable.attr('class', 'table table-striped');
      },
    });
  });


  var dashletHeader = $('#docusignMain').parent().parent().parent().parent();

  // the DS dashlet refresh event
  dashletHeader.find('a[data-dashletaction=refreshClicked]').click(function() {
    var statuses;
    $.ajax({
      type: 'GET',
      url: sugarUrl.apiRoot + '/docus_DocuSign_Status',
      headers: {
        'Content-Type': 'application/json',
        'OAuth-Token': SUGAR.App.api.getOAuthToken(),
      },
      data: { max_num: 9999 },
      async: false,
      success: function(data) {
        statuses = data;
      },
    });

    docusignLoadStatuses(statuses, sugarUrl, sugarUserId);

    SUGAR.App.utils.doWhen(function() {
      return dashletHeader.find('i[data-action=loading]').attr('class') === 'icon-refresh icon-spin';
    }, function() { // then do this
      setTimeout(function() {
        dashletHeader.find('i.icon-refresh').attr('class', 'icon-cog'); // revert icon to original
      }, 1000);
    });
  });


  // when on the Home page, don't allow DS dashlet view to change
  if (sugarUrl.isAtHome) {
    $('#docusignChart').off('click');
  }


  // move status icons upward to make room for bottom bar
  $('#docusignMain > table').css({
    position: 'relative',
    bottom: '10px',
  });

  // insert the bottom banner bar
  var widget = $('#docusignMain').parent().parent().parent();
  widget.css('overflow', 'hidden');
  var footer = $('<div id="docusignFooter"></div>').appendTo(widget);
  footer.css({
    height: '28px',
    'background-color': '#f3f3f3',
    'border-top': '1px solid #e8e8e8',
    'border-bottom-left-radius': '8px',
    'border-bottom-right-radius': '8px',
  });
  if (!sugarUrl.isAtHome) {
    footer.css({
      position: 'relative',
      bottom: '24px',
    });
  }
  var icon = $('<img src="custom/docusign/docusign.png" />').appendTo(footer);
  icon.css({
    'margin-top': '5px',
    'margin-left': '10px',
  });
}


})(jQuery);
