/*
 * DocusignSugarUrl class
 *
 * This is a utility class to parse the browser's URL into pieces to be used by
 * various parts of the DocuSign dashlet code.
 *
 * this.root:       The location of the Sugar instance
 *                  (e.g. 'http://localhost/ent/sugarcrm/').
 * this.recordType: The type of module/tab (e.g. 'Opportunities').
 * this.recordId:   The record ID (e.g. 'fcdf0fc8-7a95-f198-8692-52432435eca1').
 * this.is*:        Any property with this prefix designates a boolean.
 * this.api*:       Any property with this prefix designates a value used for
 *                  Sugar API calls.
 */
function DocusignSugarUrl(address) {
  this.isBwc = /#bwc\//.test(address); // are we on the old Sugar UI?

  var m = this.isBwc ? address.match(/^(.*?)(?:index.php)?#bwc\/.*?module=(.*?)&(?:.*?record=([^&]+))?/) : address.match(/^(.*?)(?:index.php)?#(.*?)(?:\/|$)(.*?)$/);

  this.root = m[1];
  this.recordType = m[2];
  this.recordId = m[3];

  this.apiRoot = this.root + 'rest/v10';
  this.apiParentRecord = this.apiRoot + '/' + this.recordType + '/' + this.recordId;

  this.isListing = this.recordId === '';
  this.isAtHome = this.recordType === 'Home';

  // this method returns all DS statuses relevant to where the user is in Sugar
  this.isStatusOurs = function(sugarUserId, dsStatus) {
    if (this.isAtHome) {
      // return all statuses owned by this user
      return sugarUserId === dsStatus.created_by;
    } else if (this.isListing) {
      // return statuses related to this record type owned by this user
      return sugarUserId === dsStatus.created_by &&
             this.recordType === dsStatus.docusign_parentRecord;
    } else { // this is an individual record
      // return all statuses related to this record
      return this.recordId === dsStatus.docusign_parentId;
    }
  };
}
/*// MOCHA
exports.SugarUrl = DocusignSugarUrl;
*/// MOCHA
