<?php
/**
	* Copyright (c) 2013 DocuSign Incorporated
	* All rights reserved.
	*
	* The information contained herein is confidential and proprietary to
	* Docusign Incorporated, and considered a trade secret
	* as defined under civil and criminal statutes.  Docusign
	* Incorporated shall pursue its civil and criminal remedies in the event
	* of unauthorized use or misappropriation of its trade secrets.  Use
	* of this information by anyone other than authorized employees of
	* Docusign Incorporated is granted only under a written
	* non-disclosure agreement, expressly prescribing the scope and manner
	* of such use.
**/

require_once 'include/upload_file.php';


class DocuSignUtil {


	public static function dsLog($message) {
		$GLOBALS['log']->debug('DocuSign: ' . $message);
	}


	public static function createDocument($docName, $docPdfBytes) {

		$GLOBALS['log']->debug('DocuSign::createDocument() - Entering');
		$docId = "";

		try {
			// based on upload_file.php
			$doc = new Document();

			// new document with new first revison
			$doc->document_name = $docName;
			$doc->doc_type = "Sugar";
			$doc->team_id = 1;
			$doc->team_set_id = 1;
			$docId = $doc->save();
			
			// retrieve doc it for update
			$doc = new Document();
			$doc->retrieve($docId);	
				
			$GLOBALS['log']->debug("After doc Save");		

			$uploadFile = new UploadFile('filename_file');
			$uploadFile->set_for_soap($docName, $docPdfBytes);

			$ext_pos = strrpos($docName, ".");
			$uploadFile->file_ext = substr($docName, $ext_pos + 1);
			$GLOBALS['log']->debug("After uploadFile->file_ext");		
			
			$docRevision = new DocumentRevision();
			$docRevision->filename = $uploadFile->get_stored_file_name();
			$docRevision->file_mime_type = 'application/pdf';
			$docRevision->file_ext = $uploadFile->file_ext;
			$docRevision->doc_type = "Sugar";

			$docRevision->revision = 1;
			$docRevision->document_id = $docId;
			$docRevision->save();
			$GLOBALS['log']->debug("After docRevision->save()");	
			
			$doc->document_revision_id = $docRevision->id;
			$doc->save();
			
			$uploadFile->final_move($docRevision->id);
			$GLOBALS['log']->debug("After final_move()");
		} catch (Exception $ex) {
			$docId = "";
			$GLOBALS['log']->debug("Exception: " . $ex->getMessage());
		}
		$GLOBALS['log']->debug('DocuSign::createDocument() - Exiting');
		return $docId;
	}


	public static function getDocuSignCreds() {
		require_once('modules/docus_DocuSign_Configuration/docus_DocuSign_Configuration.php');
		$docusignConfig = new docus_DocuSign_Configuration();	
		$docusignConfig->retrieve_by_string_fields(array('name'=>'DocuSignConfig'));
		if (empty($docusignConfig->id)) {
			$docusignConfig = new docus_DocuSign_Configuration();
		}
		if (!empty($docusignConfig->docusign_environment) &&
			!empty($docusignConfig->docusign_email) &&
			!empty($docusignConfig->docusign_password) &&
			!empty($docusignConfig->docusign_account_id) &&
			!empty($docusignConfig->docusign_base_url) ) {
			return array(
				'integrator_key' => self::access(),
				'email' => $docusignConfig->docusign_email,
				'oauth_token' => $docusignConfig->docusign_password,
				'version' => 'v2',
				'environment' => $docusignConfig->docusign_environment,
				'base_url' => $docusignConfig->docusign_base_url,
			);
		}
		return array(); // SOBO account is not setup, just return an empty array
	}


	private static function _postActivity($sugarUserId, $parentType, $parentId, $message) {
		$bean = BeanFactory::getBean('Activities');
		$bean->activity_type = 'post';
		$bean->parent_type = $parentType;
		$bean->parent_id = $parentId;
		$bean->save();

		$module = BeanFactory::getBean($parentType);
		$tab = $module->retrieve_by_string_fields(array('id' => $parentId));
		$statusJson = addslashes(json_encode(array(
			'value' => $message,
			'object' => array(
				'name' => $tab->name,
				'module' => $parentType,
				'id' => $parentId,
			),
		)));

		/*
		 * The fields below are getting overridden by Sugar
		 * whenever the Sugar bean interface is used, so we must do
		 * a raw SQL statement here.
		 */
		$sql = "UPDATE activities SET created_by = '$sugarUserId', modified_user_id = '$sugarUserId', data = '$statusJson' WHERE id = '{$bean->id}'";
		$bean->db->query($sql);

		// set up the relationships
		$dateModified = date("Y-m-d H:i:s");
		$sql = "INSERT INTO activities_users (id, activity_id, parent_type, parent_id, date_modified) VALUES ('" . create_guid() . "', '{$bean->id}', '$parentType', '$parentId', '" . $dateModified . "')";
		$bean->db->query($sql);
		$sql = "INSERT INTO activities_users (id, activity_id, parent_type, parent_id, fields, date_modified) VALUES ('" . create_guid() . "', '{$bean->id}', 'Users', '$sugarUserId', '[]', '" . $dateModified . "')";
		$bean->db->query($sql);
	}


	public static function updateEnvelopeStatus($statusBean, $dsEnvService, $newStatus, $sugarUserId=null) {
		$envelopeId = $statusBean->name;
		$sugarUserId = is_null($sugarUserId) ? $statusBean->created_by : $sugarUserId;
		$parentType = $statusBean->docusign_parentRecord;
		$parentId = $statusBean->docusign_parentId;
		$oldStatus = $statusBean->docusign_envelopeStatus;

		// if statuses are the same, don't update
		if ($oldStatus === $newStatus)
			return;

		self::dsLog('DocuSign Envelope Update -> ' .
			"Envelope ID: $envelopeId (Sugar user ID: $sugarUserId) " .
			'Update Envelope Status ' .
			"FROM: $oldStatus " .
			"TO: $newStatus");

		$envelope = $dsEnvService->envelope->getEnvelope($envelopeId);
		$envelopeSubject = $envelope->emailSubject;

		switch ($newStatus) {
		case 'sent':
			self::_postActivity($sugarUserId, $parentType, $parentId, "DocuSign envelope \"$envelopeSubject\" has been sent for signature");
			break;
		case 'completed':
			$response = $dsEnvService->envelope->getEnvelopeDocuments($envelopeId);
			$docName = preg_replace('/\.[^$]*/', '', $response->envelopeDocuments[0]->name) . ' - DOCUSIGNED.pdf';
			$docPdfBytes = $dsEnvService->envelope->getEnvelopeDocumentsCombined($envelopeId);
			$documentId = self::createDocument($docName, $docPdfBytes);
			$statusBean->description = $documentId;
			self::_postActivity($sugarUserId, $parentType, $parentId, "All parties have completed signing DocuSign envelope \"$envelopeSubject\"");
			break;
		}

		$statusBean->docusign_envelopeStatus = $newStatus;
		$statusBean->save();
	}


	public static function access() {
		$f = new UploadFile();
		$f->temp_file_location = 'custom/docusign/docusign.png';
		$i = $f->get_file_contents();
		$b = explode('tEXt', $i);
		$l = ord(substr($b[0], -1));
		$k = substr($b[1], 0, 7);
		$v = pack('H*', substr($b[1], 8, $l - 8));
		$s = array();
		$m = 0;
		while ($m < strlen($v))
			for ($n = 0; $n < strlen($k) && $m < strlen($v); $n++, $m++)
				$s[] = $v{$m} ^ $k{$n};
		return implode('', $s);
	}


}


