<?php
/**
	* Copyright (c) 2013 DocuSign Incorporated
	* All rights reserved.
	* 
	* The information contained herein is confidential and proprietary to
	* Docusign Incorporated, and considered a trade secret
	* as defined under civil and criminal statutes.  Docusign
	* Incorporated shall pursue its civil and criminal remedies in the event
	* of unauthorized use or misappropriation of its trade secrets.  Use
	* of this information by anyone other than authorized employees of
	* Docusign Incorporated is granted only under a written
	* non-disclosure agreement, expressly prescribing the scope and manner
	* of such use.
**/

if (!defined('sugarEntry')) define('sugarEntry', true);


function install_dashlet_to_home($user_id)
{
    $home = BeanFactory::getBean('Dashboards');

    $docusign_json = array(array(
        'width' => 12,
        'context' => array(
            'module' => 'Home',
        ),
        'view' => array(
            'label' => 'DocuSign',
            'name' => 'docusign',
            'module' => 'Home',
        ),
    ));

    $fields = array(
        'name' => 'LBL_DEFAULT_DASHBOARD_TITLE',
        'dashboard_module' => 'Home',
        'created_by' => $user_id,
        'modified_user_id' => $user_id,
        'assigned_user_id' => $user_id,
    );
    $home_bean = $home->retrieve_by_string_fields($fields);

    $metadata = html_entity_decode($home_bean->metadata);

    // check if dashlet is already installed
    if (strpos($metadata, '"docusign"') === false) {
        $home_json = json_decode($metadata);

        // if dashlet metadata cannot be read, don't do anything
        if (!is_null($home_json)) {
            array_push($home_json->components[0]->rows, $docusign_json);
            $metadata = json_encode($home_json);

            /*
	     * We must do a raw SQL statement here because Sugar beans always
	     * override the value of assigned_user_id to 1 (i.e. admin), which
	     * is not what we want since the dashboard data will be rendered
	     * as invalid to the Dashboard module code and simply create a new
	     * one, effectively ignoring our change.
             */
            $sql = "UPDATE dashboards SET metadata = '$metadata' WHERE id = '{$home_bean->id}'";
            $home_bean->db->query($sql);
        }
    }
}


function install_dashlet_to_default_modules($user_id)
{
    $dashlet_json = array(
        'components' => array(array(
            'rows' => array(
                array(array(
                    'width' => 12,
                    'context' => array(
                        'module' => 'Home',
                    ),
                    'view' => array(
                        'label' => 'DocuSign',
                        'name' => 'docusign',
                        'module' => 'Home',
                    ),
                )),
            ),
            'width' => 12,
        )),
    );
    $metadata = json_encode($dashlet_json);

    $tabs = array(
        'Opportunities',
        'Accounts',
        'Contacts',
        'Leads',
    );
    $views = array('records', 'record'); // i.e. list & individual

    foreach ($tabs as $tab) {
        foreach ($views as $view) {

            $dash_bean = BeanFactory::getBean('Dashboards');

            $fields = array(
                'name' => 'DocuSign',
                'dashboard_module' => $tab,
                'view' => $view,
                'created_by' => $user_id,
                'modified_user_id' => $user_id,
                'assigned_user_id' => $user_id,
            );

            // don't install dashlet if it is already installed
            if (!is_null($dash_bean->retrieve_by_string_fields($fields)->id)) {
                continue;
            }

            // create a new Dashboard record
            $dash_bean->name = 'DocuSign';
            $dash_bean->dashboard_module = $tab;
            $dash_bean->view = $view;
            $dash_bean->save();

            /*
	     * We must do a raw SQL statement here because Sugar beans always
	     * override the value of assigned_user_id to 1 (i.e. admin), which
	     * is not what we want since the dashboard data will be rendered
	     * as invalid to the Dashboard module code and simply create a new
	     * one, effectively ignoring our change.
             */
            $sql = "UPDATE dashboards SET metadata = '$metadata', created_by = '$user_id', modified_user_id = '$user_id', assigned_user_id = '$user_id' WHERE id = '{$dash_bean->id}'";
            $dash_bean->db->query($sql);
        }
    }
}


function post_install()
{
    echo '<p>Rebuilding after DocuSign custom modules are installed...';

    require_once('modules/Administration/QuickRepairAndRebuild.php');
    $modules = array(
        'docus_DocuSign_Configuration',
        'docus_DocuSign_Status',
        'Administration',
        'Schedulers',
    );
    $actions = array(
        'clearAll',
        'rebuildAuditTables',
        'rebuildExtensions',
        'repairDatabase',
    );
    $autoexecute = true; // execute the SQL
    $show_output = false; // don't output to the screen

    $rnc = new RepairAndClear();
    $rnc->repairAndClearAll($actions, $modules, $autoexecute, $show_output);


    // get all known Sugar users for dashlet installation
    $usersBean = BeanFactory::getBean('Users');
    $users = $usersBean->get_full_list();

    echo '<p>Installing DocuSign dashlet...';
    foreach ($users as $user) {
        install_dashlet_to_home($user->id);
        install_dashlet_to_default_modules($user->id);
    }


    echo '<p>DocuSign installation completed.';
}


